package lib

import (
	"errors"
	"fmt"
	"os"
	"strings"

	"git.ymnuktech.ru/ymnuk/yt-cli/structs"

	"github.com/schollz/progressbar/v3"
	"gopkg.in/yaml.v3"
)

func ConfigUpload(filename string) {
	if _, err := os.Stat(filename); errors.Is(err, os.ErrNotExist) {
		panic(fmt.Errorf("file not found"))
	} else if err != nil {
		panic(err)
	}
	var payload []byte
	var err error
	if payload, err = os.ReadFile(filename); err != nil {
		panic(err)
	}

	var forUpload structs.ForUpload

	if err = yaml.Unmarshal(payload, &forUpload); err != nil {
		panic(err)
	}

	if len(forUpload.Params) > 0 {
		bar := progressbar.Default(int64(len(forUpload.Params)), "Params")
		for _, parval := range forUpload.Params {
			if err = ConfigSet(parval.Name, parval.Value); err != nil {
				return
			}
			bar.Add(1)
		}
	}

	if len(forUpload.Static) > 0 {
		for _, statics := range forUpload.Static {
			if len(statics.Paths) > 0 {
				for _, path := range statics.Paths {
					if len(path.Files) > 0 {
						bar := progressbar.Default(int64(len(path.Files)), "Files")
						for _, filename := range path.Files {
							GatewayStaticFileSet(filename, statics.Domain, strings.ReplaceAll(fmt.Sprintf(`%s/%s`, path.Path, filename), "//", "/"))
							bar.Add(1)
						}
					}
					if len(path.Dirs) > 0 {
						for _, dirname := range path.Dirs {
							GatewayStaticDirSet(statics.Domain, dirname, path.Path)
						}
					}
				}
			}
		}
	}

	if len(forUpload.Functions) > 0 {
		for _, domains := range forUpload.Functions {
			if len(domains.Paths) > 0 {
				bar := progressbar.Default(int64(len(domains.Paths)), fmt.Sprintf(`Functions: %s`, domains.Domain))
				for _, functionPath := range domains.Paths {
					if functionPath.Methods.Get != nil {
						GatewayFuncSet(domains.Domain, "GET", *functionPath.Methods.Get, functionPath.Path)
					}
					if functionPath.Methods.Head != nil {
						GatewayFuncSet(domains.Domain, "HEAD", *functionPath.Methods.Head, functionPath.Path)
					}
					if functionPath.Methods.Post != nil {
						GatewayFuncSet(domains.Domain, "POST", *functionPath.Methods.Post, functionPath.Path)
					}
					if functionPath.Methods.Put != nil {
						GatewayFuncSet(domains.Domain, "PUT", *functionPath.Methods.Put, functionPath.Path)
					}
					if functionPath.Methods.Delete != nil {
						GatewayFuncSet(domains.Domain, "DELETE", *functionPath.Methods.Delete, functionPath.Path)
					}
					if functionPath.Methods.Connect != nil {
						GatewayFuncSet(domains.Domain, "CONNECT", *functionPath.Methods.Connect, functionPath.Path)
					}
					if functionPath.Methods.Options != nil {
						GatewayFuncSet(domains.Domain, "OPTIONS", *functionPath.Methods.Options, functionPath.Path)
					}
					if functionPath.Methods.Trace != nil {
						GatewayFuncSet(domains.Domain, "TRACE", *functionPath.Methods.Trace, functionPath.Path)
					}
					if functionPath.Methods.Patch != nil {
						GatewayFuncSet(domains.Domain, "PATCH", *functionPath.Methods.Patch, functionPath.Path)
					}
					bar.Add(1)
				}
			}
		}
	}

	if len(forUpload.Proxy) > 0 {
		for _, proxies := range forUpload.Proxy {
			if len(proxies.Paths) > 0 {
				bar := progressbar.Default(int64(len(proxies.Paths)), fmt.Sprintf(`Proxies: %s`, proxies.Domain))
				for _, path := range proxies.Paths {
					GatewayProxySet(proxies.Domain, path.Redirect, path.Path)
					bar.Add(1)
				}
			}
		}
	}

	if forUpload.Security != nil {
		SecuritySet(forUpload.Security)
	}

	if len(forUpload.Tokens) > 0 {
		for _, tokens := range forUpload.Tokens {
			bar := progressbar.Default(int64(len(tokens.Paths)), fmt.Sprintf(`Tokens: %s`, tokens.Domain))
			for _, path := range tokens.Paths {
				SetToken(tokens.Domain, path)
				bar.Add(1)
			}
		}
	}
}
