package structs

type ForUpload struct {
	Functions []Function `json:"functions,omitempty"`
	Proxy     []Proxy    `json:"proxy,omitempty"`
	Static    []Static   `json:"static,omitempty"`
	Security  *Security  `json:"security,omitempty"`
	Params    []Param    `json:"params,omitempty"`
	Tokens    []Token    `json:"token,omitempty"`
}

type Token struct {
	Domain string   `json:"domain"`
	Paths  []string `json:"paths"`
}

type Function struct {
	Domain string         `json:"domain"`
	Paths  []FunctionPath `json:"paths"`
}

type FunctionPath struct {
	Path    string  `json:"path"`
	Methods Methods `json:"methods"`
}

type Methods struct {
	Get     *string `json:"get,omitempty"`
	Head    *string `json:"head,omitempty"`
	Post    *string `json:"post,omitempty"`
	Put     *string `json:"put,omitempty"`
	Delete  *string `json:"delete,omitempty"`
	Connect *string `json:"connect,omitempty"`
	Options *string `json:"options,omitempty"`
	Trace   *string `json:"trace,omitempty"`
	Patch   *string `json:"patch,omitempty"`
}

type Param struct {
	Name  string `json:"name"`
	Value string `json:"value"`
}

type Proxy struct {
	Domain string      `json:"domain"`
	Paths  []ProxyPath `json:"paths"`
}

type ProxyPath struct {
	Path     string `json:"path"`
	Redirect string `json:"redirect"`
}

type Security struct {
	Uri      *string `json:"uri,omitempty"`
	Redirect *string `json:"redirect,omitempty"`
	Func     *string `json:"func,omitempty"`
	Rbac     *Rbac   `json:"rbac,omitempty"`
	Users    []User  `json:"users,omitempty"`
}

type User struct {
	Login    string  `json:"login,omitempty"`
	Password string  `json:"password,omitempty"`
	Email    string  `json:"email,omitempty"`
	Roles    []Role  `json:"roles,omitempty"`
	Groups   []Group `json:"groups,omitempty"`
}

type Rbac struct {
	Roles  []Role  `json:"roles,omitempty"`
	Groups []Group `json:"groups,omitempty"`
}

type Role struct {
	Name        string  `json:"name,omitempty"`
	Display     *string `json:"display,omitempty"`
	Description *string `json:"description,omitempty"`
	Access      *string `json:"access,omitempty"` // Указывается как permit либо deny. Если пустое, то permit. Применяется только в группах и при назначении прав пользователю
}

type Group struct {
	Name        string  `json:"name"`
	Display     *string `json:"display"`
	Roles       []Role  `json:"roles,omitempty"`
	Description *string `json:"description,omitempty"`
	Access      *string `json:"access,omitempty"` // Указывается как permit либо deny. Если пустое, то permit. Применяется только в группах и при назначении прав пользователю
}

type Static struct {
	Domain string       `json:"domain"`
	Paths  []StaticPath `json:"paths"`
}

type StaticPath struct {
	Path  string   `json:"path"`
	Files []string `json:"files,omitempty"`
	Dirs  []string `json:"dirs,omitempty"`
}
